/**
 * ExCons
 * 
 * This program demonstrates a producer-consumer problem
 * 
 * there are 10 producers and 10 consumers.
 * each producer produces a sequence of integer numbers
 * from 1 to 10 and pushes them to the common TCP/IP channel
 * each consumer takes 10 integer numbers from the channel
 * and computes their sum.
 * (jdk 1.1)
 */

import java.io.*;

public class ExCons {
  public static void main(String[] args) {
    ChannelFactory chanfac = new ChannelFactory();
    new ExCons().work(new PrintWriter(System.out,true),chanfac);
  }

  /**
   * main
   */
  void work(PrintWriter out, ChannelFactory cf) {
    int cworkers = 10;
    int count = 10;

    ChannelFactory chanfac = cf;
    SocketChannel rchan = null;
    try { rchan = chanfac.getChannel(); }
    catch (IOException e) {
           System.err.println("Can not get Channel "+e); }

    ExProConRes res = new ExProConRes();

    // creating consumers
    Thread ct[] = new Thread[cworkers];
    for (int i=0; i<cworkers; i++) {
      ct[i] = new Thread(new ExProConCons(rchan, i, count, res, out));
      ct[i].start();
    }

    // terminating
    for (int i=0; i<cworkers; i++) 
      try { ct[i].join(); } catch (InterruptedException e) {}

    out.println("result is "+res.get());
    out.println("terminated");
  }
}

/*
 * Consumer part
 */
class ExProConCons implements Runnable {
  SocketChannel rchan;
  int id;
  int max;
  ExProConRes res;
  PrintWriter out;

  /**
   * Constructs a producer unit
   */
  public ExProConCons(SocketChannel c, int i, int m,
		      ExProConRes r, PrintWriter o) {
    rchan = c;
    id = i;
    max = m;
    res = r;
    out = o;
  }

  
  /**
   * Performs the consumer's work
   */
  public void run() {
    int sum = 0;

    out.println("cons "+id+" started");

    Object x;
    for (int i=1; i<=max; i++) {
      doSomeWork(2);

      try { x = rchan.receive(); }
      catch (IOException e) {
             System.err.println(e); return; }
      catch (ClassNotFoundException e) {
             System.err.println(e); return; }

      if (x instanceof Integer) { 
         sum += ((Integer)x).intValue(); }
    }

    res.add(sum);

    out.println("cons "+id+" finished ; sum = "+sum);
  }

  // simulates some computation
  // in reality, it idles for a random duration up to t seconds
  static void doSomeWork(double t) {
    try {
      Thread.currentThread().sleep((long)(t*1000.0*Math.random()));
    }
    catch (InterruptedException e) {
    }
  }
}

