/**
 * ExProd
 * 
 * This program demonstrates a producer-consumer problem
 * 
 * there are 10 producers and 10 consumers.
 * each producer produces a sequence of integer numbers
 * from 1 to 10 and pushes them to the common TCP/IP channel
 * each consumer takes 10 integer numbers from the channel
 * and computes their sum.
 * (jdk 1.1)
 */

import java.io.*;

public class ExProd {
  public static void main(String[] args) {
    ChannelFactory chanfac = new ChannelFactory();
    new ExProd().work(new PrintWriter(System.out,true),chanfac);
  }

  /**
   * main
   */
  void work(PrintWriter out, ChannelFactory cf) {
    int pworkers = 10;
    int count = 10;

    ChannelFactory chanfac = cf;
    SocketChannel rchan = null;
    try { rchan = chanfac.getChannel("localhost",4711); }
    catch (IOException e) {
           System.err.println("Can not get Channel "+e); }

    // creating producers
    Thread pt[] = new Thread[pworkers];
    for (int i=0; i<pworkers; i++) {
      pt[i] = new Thread(new ExProConProd(rchan, i, count, out));
      pt[i].start();
    }

    // terminating
    for (int i=0; i<pworkers; i++) 
      try { pt[i].join(); } catch (InterruptedException e) {}

    int ans = pworkers*count*(count+1)/2;
    out.println("result should be "+ans);
    out.println("terminated");
  }
}

/*
 * Producer part
 */
class ExProConProd implements Runnable {
  SocketChannel rchan;
  int id;
  int max;
  PrintWriter out;

  /**
   * Constructs a producer unit
   */
  public ExProConProd(SocketChannel c, int i, int m, PrintWriter o) {
    rchan = c;
    id = i;
    max = m;
    out = o;
  }

  /**
   * Performs the producer's work
   */
  public void run() {

    out.println("prod "+id+" started");

    for (int i=1; i<=max; i++) {
      doSomeWork(2);

      try { rchan.send( new Integer(i) ); }
      catch (IOException e) {
             System.err.println(e); return; }

    }

    out.println("prod "+id+" finished");
  }

  // simulates some computation
  // in reality, it idles for a random duration up to t seconds
  static void doSomeWork(double t) {
    try {
      Thread.currentThread().sleep((long)(t*1000.0*Math.random()));
    }
    catch (InterruptedException e) {
    }
  }
}


